/* Generate a diffusion limited aggregate. */

#include <stdio.h>
#include <math.h>
#include <gl.h>
#include "rand.h"

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#define DLASIZE 512
#define SIZE (DLASIZE + 2)      /* borders */
#define TOTAL 5000
#define BORDER_ZONE 40          /* border ring around the DLA */

char Dla[SIZE][SIZE];

#define ON 1
#define OFF 0

#define alone(i, j) (Dla[i][j - 1] == OFF && Dla[i][j + 1] == OFF && \
		     Dla[i - 1][j] == OFF && Dla[i + 1][j] == OFF)

/* These define the directions that we can move. */

int Diri[] = { 0,  1,  0, -1 };
int Dirj[] = { 1,  0, -1,  0 };

main()
{
	int n, i, j, k;
	int radius;     /* This keeps track of the radius of the dla. */
	int x, y, d;
	int fell;
	FILE *f;

	/* Init the random number generator. */

	init_rand();
foreground();
prefsize(DLASIZE, DLASIZE);
winopen("");
ortho2(1, DLASIZE, 1, DLASIZE);
color(WHITE);
clear();
color(BLACK);

	/* Init the array - drop a dot in the center. */

	for (i = 0; i < SIZE; i++) {
		for (j = 0; j < SIZE; j++) {
			Dla[i][j] = OFF;
		}
	}
	Dla[SIZE/2][SIZE/2] = ON;
pnt2(SIZE/2, SIZE/2);
	radius = BORDER_ZONE;

	/* Start building the DLA. */

	n = 0;
	while (n < TOTAL) {

		/* Drop a dot out at the border.  Pick two (x, y) such that
		|x| + |y| == the distance to the border from the center. */

		x = rnd(radius);
		y = radius - x - 1;
		if (rnd(2) == 1) {
			x = -x;
		}
		if (rnd(2) == 1) {
			y = -y;
		}
		j = SIZE/2 + x;
		i = SIZE/2 + y;

		/* Now the dot walks randomly until it's next to another dot. */

		fell = FALSE;
		while (alone(i, j)) {
			k = rnd(4);
			i += Diri[k];
			j += Dirj[k];
			if (i < 1 || i > DLASIZE || j < 1 || j > DLASIZE) {
				fell = TRUE;
				break;
			}
			x = (j - SIZE/2);
			y = (i - SIZE/2);
			if (_ABS(x) + _ABS(y) > radius) {
				fell = TRUE;
				break;
			}
		}

		if (fell) {

			/* Fell off the edge of the universe. */

			continue;
		}

		/* Plant this dot here and figure the new radius. */

		Dla[i][j] = ON;
pnt2(j, i);
		x = (j - SIZE/2);
		y = (i - SIZE/2);
		d = _ABS(x) + _ABS(y) + BORDER_ZONE;
		if (d > radius) {
			radius = d;
		}
		n++;
		fprintf(stderr, "%d\n", n);
		if (radius >= DLASIZE / 2) {
			break;
		}
	}

	f = fopen("dla.dat", "wt");
	for (i = 1; i <= DLASIZE; i++) {
		for (j = 1; j <= DLASIZE; j++) {
			if (Dla[i][j] == ON) {
				fprintf(f, "%d %d\n", j, i);
			}
		}
	}
	fclose(f);
}
